"""Overlay module."""

from abc import ABC
from typing import Iterable, cast

from lxml import etree  # type: ignore

from pyLiveKML.objects.AbstractView import AbstractView
from pyLiveKML.objects.Feature import Feature
from pyLiveKML.objects.Object import (
    _BaseObject,
    _ChildDef,
    _FieldDef,
    ColorParse,
    NoDump,
)
from pyLiveKML.objects.Region import Region
from pyLiveKML.objects.StyleSelector import StyleSelector
from pyLiveKML.objects.TimePrimitive import TimePrimitive
from pyLiveKML.types import GeoColor


class _Overlay_Icon(_BaseObject):
    """A minimalist Icon class, used only within `Overlay`."""

    _kml_tag = "Icon"
    _kml_fields = _BaseObject._kml_fields + (_FieldDef("href"),)

    def __init__(self, href: str):
        """_Overlay_Icon instance constructor."""
        super().__init__()
        self.href = href


class Overlay(Feature, ABC):
    """A KML 'Overlay' tag constructor.

    This is an abstract element and cannot be used directly in a KML file. `Overlay` is
    the base type for image overlays drawn on the planet surface or on the screen.
    `icon` specifies the image to use and can be configured to reload images based on a
    timer or by camera changes. This class also includes specifications for stacking
    order of multiple overlays and for adding color and transparency values to the base
    image.

    References
    ----------
    * https://developers.google.com/kml/documentation/kmlreference#overlay

    Parameters
    ----------
    icon : str | None, default = None
        Defines the location of the image to be used as the `Overlay`. This location can
        be either on a local file system or on a web server. If this attribute is `None`,
        a rectangle is drawn using the color and size defined by the ground or screen
        overlay.
    color : GeoColor | int | None, default = None
        The image or rectangle color and transparency.
    draw_order : int | None, default = None
        Defines the stacking order for the images in overlapping `Overlay`s. `Overlay`s
        with higher `draw_order` values are drawn on top of `Overlay`s with lower
        `draw_order` values.
    name : str|None, default = None
        User-defined text displayed in the 3D viewer as the label for the object.
    visibility : bool | None, default = None
        Specifies whether the `Feature` is drawn in the 3D viewer when it is initially
        loaded. In order for a `Feature` to be visible, the `<visibility>` tag of all
        its ancestors must also be set `True`.
    is_open : bool | None, default = None
        Specifies whether a `Document` or `Folder` appears closed or open when first
        loaded into the "Places" panel. `False` or `None` is collapsed (the default),
        `True` is expanded. This element applies only to `Document`, `Folder`, and
        `NetworkLink`.
    author_name : str | None, default = None
        The name of the author of the `Feature`.
    author_link : str | None, default = None
        URL of the web page containing the KML file.
    address : str | None, default = None
        A string value representing an unstructured address written as a standard street,
        city, state address, and/or as a postal code.
    phone_number : str | None, default = None
        A string value representing a telephone number. This element is used by Google
        Maps Mobile only. The industry standard for Java-enabled cellular phones is
        RFC2806.
    snippet : str | None, default = None
        A short description of the `Feature`. In Google Earth, this description is
        displayed in the "Places" panel under the name of the `Feature`. If a `<Snippet>`
        is not supplied, the first two lines of the `<description>` are used. In Google
        Earth, if a `Placemark` contains both a `<description>` and a `<Snippet>`, the
        `<Snippet>` appears beneath the `Placemark` in the "Places" panel, and the
        `<description>` appears in the `Placemark`'s description balloon. This tag does
        not support HTML markup.
    snippet_max_lines : int | None, default = None
    description : str | None, default = None
        User-supplied content that appears in the description balloon. HTML *is*
        supported, but it is **highly** recommended to read the detailed documentation
        at
        https://developers.google.com/kml/documentation/kmlreference#elements-specific-to-feature
    abstract_view : AbstractView | None, default = None
        Any concrete subclass of :class:`pyLiveKML.objects.AbstractView`, i.e. either a
        :class:`pyLiveKML.objects.Camera` or :class:`pyLiveKML.objects.LookAt`
    time_primitive : TimePrimitive | None, default = None
        Any concrete subclass of :class:`pyLiveKML.objects.TimePrimitive`, i.e. either a
        :class:`pyLiveKML.objects.TimeStamp` or :class:`pyLiveKML.objects.TimeSpan`
    style_url : str | None = None
        URL of a `<Style>` or `<StyleMap>` defined in a `<Document>`. If the style is in
        the same file, use a # reference. If the style is defined in an external file,
        use a full URL along with # referencing.
    styles : StyleSelector | Iterable[StyleSelector] | None, default = None
        One or more `Style`s and `StyleMap`s can be defined to customize the appearance
        of any element derived from `Feature` or of the `Geometry` in a `Placemark`. A
        style defined within a `Feature` is called an "inline style" and applies only to
        the `Feature` that contains it. A style defined as the child of a `<Document>` is
        called a "shared style." A shared style must have an id defined for it. This id
        is referenced by one or more `Features` within the `<Document>`. In cases where
        a style element is defined both in a shared style and in an inline style for a
        `Feature` — that is, a `Folder`, `GroundOverlay`, `NetworkLink`, `Placemark`, or
        `ScreenOverlay` — the value for the `Feature`'s inline style takes precedence over
        the value for the shared style.
    region : Region | None, default = None
        `Feature`s and `Geometry`'s associated with a `Region` are drawn only when the
        `Region` is active.

    Attributes
    ----------
    Same as parameters.

    """

    _kml_tag = ""
    _kml_fields: tuple[_FieldDef, ...] = Feature._kml_fields + (
        _FieldDef("icon", dumper=NoDump),
        _FieldDef("color", parser=ColorParse),
        _FieldDef("draw_order", "drawOrder"),
    )
    _kml_children = Feature._kml_children + (_ChildDef("icon"),)

    def __init__(
        self,
        icon: str | None = None,
        color: GeoColor | int | None = None,
        draw_order: int | None = None,
        name: str | None = None,
        visibility: bool | None = None,
        is_open: bool | None = None,
        author_name: str | None = None,
        author_link: str | None = None,
        address: str | None = None,
        phone_number: str | None = None,
        snippet: str | None = None,
        snippet_max_lines: int | None = None,
        description: str | None = None,
        abstract_view: AbstractView | None = None,
        time_primitive: TimePrimitive | None = None,
        style_url: str | None = None,
        styles: StyleSelector | Iterable[StyleSelector] | None = None,
        region: Region | None = None,
    ):
        """Overlay instance constructor."""
        Feature.__init__(
            self,
            name=name,
            visibility=visibility,
            is_open=is_open,
            author_name=author_name,
            author_link=author_link,
            address=address,
            phone_number=phone_number,
            snippet=snippet,
            snippet_max_lines=snippet_max_lines,
            description=description,
            abstract_view=abstract_view,
            time_primitive=time_primitive,
            style_url=style_url,
            styles=styles,
            region=region,
        )
        ABC.__init__(self)
        self._icon = _Overlay_Icon(icon) if icon else None
        self.draw_order = draw_order
        self.color = cast(GeoColor | None, color)

    @property
    def icon(self) -> str | None:
        """The location of the icon image file.

        Parameters
        ----------
        value : str | None

        :return: The (optional) location of the icon image file.
        :rtype: str | None

        """
        return self._icon.href if self._icon else None

    @icon.setter
    def icon(self, value: str | None) -> None:
        if value is None:
            self._icon = None
        elif self._icon is None:
            self._icon = _Overlay_Icon(value)
        else:
            self._icon.href = value
